/* ============================================================================
 * Project Name: RMS_dsADC_wFunc_04a_basic
 *
 * Description:
 *   Demo project showing RMS calculation using window function and quadro-buffering. 
 *   Incoming AC signal is continuously sampled by DeltaSigma ADC (16-bit)
 *   and transferred to the RAM Buffer using DMA.
 *   Once 1/4 Buffer is filled, ISR is fired and Buffer data is processed.   
 *   RMS value calculated using window function (Hanning):
 *   MS = SUM(Buffer^2[i] x Window[i]) / SUM(Window[i])
 *   RMS = sqrt(MS)
 *
 *   Project points
 *   1. ADC is sampling continuously at 4 kHz (1 - 8 kHz)
 *   2. ADC sampling and AC signal are not synchronized
 *   3. DMA transfers ADC data to RAM Buffer (4xN samples, where N=201 (401, 801)
 *   4. ISR fires each 1/4 Buffer is filled to process the data
 *   5. Data processing (RMS) takes approx. 240us (debug) / 120us (release) (N=200) 
 *   6. RMS results update rate is 20 Hz (ADC clock: 4 kHz, N=200) 
 *   7. RMS group delay is 50 ms
 *
 *   Note that all custom components has been stripped from this demo.
 *   For proper operation this project requires external signal generator.
 *
 * ============================================================================
 * PROVIDED AS-IS, NO WARRANTY OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * FREE TO SHARE, USE AND MODIFY UNDER TERMS: CREATIVE COMMONS - SHARE ALIKE
 * ============================================================================
*/



#include <project.h>
#include <stdio.h> // sprintf
#include <DMA.h>
#include <RMS.h>


#define BlinkLED(); {Pin_LED_Write(1); CyDelayUs(50); Pin_LED_Write(0);} // blink LED indicator for debugging

char buff[64];      // UART output buffer            
 


//==============================================================================
// interrupt on DMA complete
//==============================================================================
volatile CYBIT flag_NrqReady = 0; 
CY_ISR(myNrqInt)                
{   
    flag_NrqReady = 1;
}



//============================================
// Forward declarations
//============================================
void DisplayData();             // output to terminal


void Initialize(void)
{   
    CyGlobalIntEnable;          //enable global interrupts.
    
    UART_1_Start();             // Start UART  
    CyDelay(100);               // 
    
    DMA1_Config();              // initialize DMA for ADC-DMA-RAM transfer
    isrNrq_StartEx(myNrqInt);   //start isrNrq interrupt (DMA1 transfer complete)

    Opamp_RMS_Start();          // output RMS voltage to scope
    VDAC_RMS_Start();           // output RMS voltage to scope
    
    ConfigRMS16();              // configure weighting function buffer (16-bit precision)
    
    ADC_1_Start();              // initialize and enable ADC
    ADC_1_IRQ_Disable();        // disable the ADC ISR as it is not required
    ADC_1_StartConvert();       // start ADC sampling
}


int main()
{
    Initialize();

    for(;;)
    {   

        if (flag_NrqReady==1)                           // RAM buffer is populated -> "process" data
        {
            flag_NrqReady=0;                            // reset flag
            BlinkLED();                                 // completion indicator

            GetRMS16();                                 // calculate RMS
            
            VDAC_RMS_SetValue( RMSV_to_VDAC(RMS_V) );   // output control voltage to scope
            DisplayData();                              // output result to terminal
        }
              
    }
    
} //main

   
//==============================================================================
// 
//==============================================================================

void DisplayData(void) // output to terminal
{   
    sprintf(buff, "RMS: %.3f V\r\n", RMS_V);   
    
    UART_1_PutString(buff);              // Send data to Terminal
}


/// END OF PROGRAM //////////////////////////////////////////////////////////////////////////////////////

